using System.Drawing;

namespace AccessibilityAnalayzer
{
    using JetBrains.ReSharper;
    using JetBrains.ReSharper.ActionManagement;
    using JetBrains.ReSharper.Component;
    using JetBrains.ReSharper.Editor;
    using JetBrains.ReSharper.Shell;
    using JetBrains.ReSharper.TextControl;
    using JetBrains.ReSharper.TextControl.Data;
    using JetBrains.ReSharper.TextControl.Markup;
    using JetBrains.ReSharper.Tips;
    using JetBrains.ReSharper.UsageHightlighting;
    using JetBrains.ReSharper.Util;
    using JetBrains.ReSharper.WindowManagement;
    using System;
    using System.Collections;
    using System.Runtime.CompilerServices;
    using System.Text;

    /// <summary>
    /// Manages highlighting text ranges in code
    /// </summary>
    public class AccessibilityHighlightingManager : UsageHighlightingManager
    {
        private static AccessibilityHighlightingManager _Instance=null;
        
        new public static AccessibilityHighlightingManager Instance
        {
            get{ 
                if(_Instance==null)
                   _Instance=new AccessibilityHighlightingManager();
               return _Instance;
            }
        }
        
        static AccessibilityHighlightingManager()
        {
            AccessibilityHighlightingManager.HIGHLIGHTERS_KEY = new Key("Accessibility Highlighting Spots");
        }

        public override void GoToNextUsage(ITextControl textControl)
        {
            if (textControl == null)
            {
                throw new ArgumentNullException("textControl");
            }
            this.GoToUsage(textControl, Direction.Forward);
        }

        public override void GoToPreviousUsage(ITextControl textControl)
        {
            if (textControl == null)
            {
                throw new ArgumentNullException("textControl");
            }
            this.GoToUsage(textControl, Direction.Backwards);
        }

        private void GoToUsage(ITextControl textControl, Direction direction)
        {
            if (textControl == null)
            {
                throw new ArgumentNullException("textControl");
            }
            TipsManager.Instance.FeatureIsUsed("HighlightUsages.NextPrevOccurence");
            ArrayList list1 = (ArrayList) textControl.Document.GetData(HIGHLIGHTERS_KEY);
            Logger.Assert(list1 != null, "highlighters != null", new object[0]);
            int num1 = textControl.CaretModel.Offset;
            IHighlighter highlighter1 = null;
            IHighlighter highlighter2 = null;
            foreach (IHighlighter highlighter3 in list1)
            {
                if (!highlighter3.IsValid)
                {
                    continue;
                }
                if (highlighter3.Range.StartOffset > num1)
                {
                    highlighter1 = highlighter3;
                    break;
                }
                if (!highlighter3.Range.Contains(num1))
                {
                    highlighter2 = highlighter3;
                }
            }
            if (direction == Direction.Backwards)
            {
                highlighter1 = highlighter2;
            }
            if (highlighter1 != null)
            {
                textControl.CaretModel.MoveTo(highlighter1.Range.StartOffset);
                textControl.ScrollingModel.ScrollToCaret(TextControlScrollType.CENTER);
            }
        }

        public override bool HasNextUsage(ITextControl textControl)
        {
            if (textControl == null)
            {
                throw new ArgumentNullException("textControl");
            }
            return true;
        }

        public override bool HasPrevUsage(ITextControl textControl)
        {
            if (textControl == null)
            {
                throw new ArgumentNullException("textControl");
            }
            return true;
        }

        public override IHighlightingHandler Highlight(ITextControl textControl, UsageHighlightingEntry[] entries, bool unhighlightOnDocumentChange)
        {
            return new HighlightingHandler(textControl, entries, unhighlightOnDocumentChange);
        }

        public override bool IsHighlightingPresent(ITextControl textControl)
        {
            if (textControl == null)
            {
                throw new ArgumentNullException("textControl");
            }
            return (textControl.Document.GetData(AccessibilityHighlightingManager.HIGHLIGHTERS_KEY) != null);
        }


        internal static readonly Key HIGHLIGHTERS_KEY;


        public enum Direction
        {
            Forward,
            Backwards
        }

        internal class HighlightingHandler : IHighlightingHandler, IHighlighterTooltip
        {
            public event EventHandler Cancelled;

            static HighlightingHandler()
            {
                AccessibilityHighlightingManager.HighlightingHandler.ourKey = new Key("AccessibilityHighlighting");
            }

            private void Document_OnDocumentChanged(DocumentChangeEventArgs d)
            {
                this.UnHighlight();
            }

            public HighlightingHandler(ITextControl textControl, UsageHighlightingEntry[] entries, bool unhighlightOnDocumentChange)
            {
                this.myHighlighters = new ArrayList();
                this.myMarkupModel = textControl.MarkupModel;
                this.myDocument = textControl.Document;
                UsageHighlightingEntry[] entryArray1 = entries;
                for (int num1 = 0; num1 < entryArray1.Length; num1++)
                {
                    AccessibilityHighlightEntry entry1 = entryArray1[num1] as AccessibilityHighlightEntry;
                    if (entry1 == null)
                        continue;
                    if (entry1.Range.Document == this.myDocument)
                    {
                        //(entry1.Type == UsageHighlightingType.Write) ? "ReSharper Write Usage" : "ReSharper Read Usage"
                        IHighlighter highlighter;
                        if(entry1.AccessibilityHighlightEntryUserData.AccessibilitySuggestion==AccessibilitySuggestion.RemoveUnused)
                            highlighter = textControl.MarkupModel.AddHighlighter(AccessibilityHighlightingManager.HighlightingHandler.ourKey, entry1.Range.TextRange, AreaType.EXACT_RANGE, 0x1387, "ReSharper Write Usage", ErrorStripeAttributes.Empty, this);
                        else
                            highlighter = textControl.MarkupModel.AddHighlighter(AccessibilityHighlightingManager.HighlightingHandler.ourKey, entry1.Range.TextRange, AreaType.EXACT_RANGE, 0x1387, "ReSharper Read Usage", ErrorStripeAttributes.Empty, this);
                        highlighter.UserData = entry1.AccessibilityHighlightEntryUserData;
                        this.myHighlighters.Add(highlighter);
                    }
                }
                ArrayList list1 = (ArrayList) this.myDocument.GetData(AccessibilityHighlightingManager.HIGHLIGHTERS_KEY);
                if (list1 == null)
                {
                    list1 = new ArrayList();
                    this.myDocument.PutData(AccessibilityHighlightingManager.HIGHLIGHTERS_KEY, list1);
                }
                list1.AddRange(this.myHighlighters);
                list1.Sort(new RangeHighlighterComparer());
                this.myEscapeHandler = new EscapeHandler(this);
                this.myEscapeHandler.Subscribe();
                this.myUnhighlightOnDocumentChange = unhighlightOnDocumentChange;
                if (this.myUnhighlightOnDocumentChange)
                {
                    this.myDocument.DocumentChanged += new DocumentChangedEventHandler(this.Document_OnDocumentChanged);
                }
            }

            public string GetTooltip(IHighlighter highlighter)
            {
                StringBuilder builder1 = new StringBuilder();
                DocumentCoords coords1 = highlighter.Document.GetCoordsByOffset(highlighter.Range.StartOffset);
                int num1 = coords1.Line;
                DocumentCoords coords2 = highlighter.Document.GetCoordsByOffset(highlighter.Range.EndOffset);
                int num2 = coords2.Line;
                for (int num3 = num1; num3 <= num2; num3++)
                {
                    if (num3 != num1)
                    {
                        builder1.Append("\r\n");
                    }
                    builder1.Append(highlighter.Document.GetLine(num3).Trim());
                }
                return builder1.ToString();
            }

            public void UnHighlight()
            {
                this.UnHighlightInternal();
                this.Unsubscribe();
            }

            private void UnHighlightInternal()
            {
                if (this.myHighlighters.Count > 0)
                {
                    ArrayList list1 = (ArrayList) this.myDocument.GetData(AccessibilityHighlightingManager.HIGHLIGHTERS_KEY);
                    Logger.Assert(list1 != null);
                    foreach (IHighlighter highlighter1 in this.myHighlighters)
                    {
                        if (list1.Contains(highlighter1))
                        {
                            this.myMarkupModel.RemoveHighlighter(highlighter1);
                            list1.Remove(highlighter1);
                        }
                    }
                    if (list1.Count == 0)
                    {
                        this.myDocument.PutData(AccessibilityHighlightingManager.HIGHLIGHTERS_KEY, null);
                    }
                    WindowManager.Instance.ClearStatusBar();
                    this.myHighlighters.Clear();
                }
            }
            
           
            private void Unsubscribe()
            {
                if (this.myUnhighlightOnDocumentChange)
                {
                    this.myDocument.DocumentChanged -= new DocumentChangedEventHandler(this.Document_OnDocumentChanged);
                }
                this.myEscapeHandler.Unsubscribe();
            }


            private IDocument myDocument;
            private EscapeHandler myEscapeHandler;
            private ArrayList myHighlighters;
            private IMarkupModel myMarkupModel;
            private bool myUnhighlightOnDocumentChange;
            private static Key ourKey;
            internal const string READ_HIGHLIGHTER_ID = "ReSharper Read Usage";
            internal const string WRITE_HIGHLIGHTER_ID = "ReSharper Write Usage";


            private class EscapeHandler : EscapeActionHandlerBase
            {
                public EscapeHandler(AccessibilityHighlightingManager.HighlightingHandler handler)
                {
                    if (handler == null)
                    {
                        throw new ArgumentNullException("handler");
                    }
                    this.myHandler = handler;
                }

                protected override void Execute(IDataContext dataContext)
                {
                    this.myHandler.UnHighlight();
                    if (this.myHandler.Cancelled != null)
                    {
                        this.myHandler.Cancelled(this, EventArgs.Empty);
                    }
                }

                protected override bool IsAvailable(IDataContext dataContext)
                {
                    ITextControl control1 = (ITextControl) dataContext.GetData(DataConstants.TEXT_CONTROL);
                    if (control1 == null)
                    {
                        return false;
                    }
                    return (this.myHandler.myMarkupModel == control1.MarkupModel);
                }


                private AccessibilityHighlightingManager.HighlightingHandler myHandler;
            }

            private class RangeHighlighterComparer : IComparer
            {
                public RangeHighlighterComparer()
                {
                    this.myRangeMarkersComparer = new RangeableComparer();
                }

                public int Compare(object x, object y)
                {
                    return this.myRangeMarkersComparer.Compare(x, y);
                }


                private RangeableComparer myRangeMarkersComparer;
            }
        }
    }
}

