using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using JetBrains.ReSharper.ActionManagement;
using JetBrains.ReSharper.CodeInsight.FindUsages;
using JetBrains.ReSharper.Editor;
using JetBrains.ReSharper.ProjectModel;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Psi.Search;
using JetBrains.ReSharper.Shell;
using JetBrains.ReSharper.Shell.Progress;
using JetBrains.ReSharper.UsageHightlighting;
using JetBrains.ReSharper.Util;

namespace AccessibilityAnalayzer
{
    internal class AccessibilityHighlightingRangeEvaluator:IHighlightingRangeEvaluator
    {
        private string statusBarText="";
        private AccessibilityHighlightEntry[] entries;
        private IDeclaredElement myDeclaredElement;
        private IProjectFile myProjectFile;
        private ISolution mySolution;
        private IProjectModelElement myProjectModelElement;

        public AccessibilityHighlightingRangeEvaluator(IDeclaredElement declaredElement, IProjectFile projectFile, ISolution solution, IProjectModelElement projectModelElement)
        {
            this.myDeclaredElement = declaredElement;
            this.myProjectFile = projectFile;
            this.mySolution=solution;
            this.myProjectModelElement = projectModelElement;
            this.Evaluate();
        }

        private void Evaluate()
        {
            ITypeMember[] members;
            ITypeElement typeElement;
            if (myDeclaredElement is ITypeElement)
            {
                typeElement = myDeclaredElement as ITypeElement;
                members = typeElement.GetMembers();
            }
            else
            {
                typeElement = myDeclaredElement.GetContainingType();
                members = typeElement.GetMembers();
            }
            ArrayList list = new ArrayList();

            object[] searchResultAsObj=FindUsages(typeElement);
            if (searchResultAsObj == null)
                return;
          
            for (int i = 0; i < members.Length; i++)
            {
                IDeclaredElement element;
                if (members[i] is IProperty)
                {
                    element = members[i] as IProperty;
                }
                else if (members[i] is IMethod)
                {
                    element = (members[i] as IMethod);
                }
                else
                    continue;
                
                IDeclaration[] declarations = element.GetDeclarations();
                IDeclaration declaration = FindDeclarationInFile(declarations);
                if (IsOverridingASystemFunction(declaration))
                    continue;
                DocumentRange propertyDeclarationDocumentRange = GetMemberDocumentRange(declaration);
                if (propertyDeclarationDocumentRange != DocumentRange.InvalidRange)
                {
                    UsageStatus usageStatus = GetUsageStatus(element, searchResultAsObj);
                    AccessibilitySuggestion accessibilitySuggestion = GetAccessibiltySuggestion(element, usageStatus);
                    if (accessibilitySuggestion != AccessibilitySuggestion.OK)
                        list.Add(new AccessibilityHighlightEntry(propertyDeclarationDocumentRange, UsageHighlightingType.Write, accessibilitySuggestion, element,declaration));
                    
                }
            }

            this.entries = (AccessibilityHighlightEntry[])list.ToArray(typeof(AccessibilityHighlightEntry));
        }

        private bool IsOverridingASystemFunction(IDeclaration declaration)
        {
           IOverridableMember overridableMember=declaration as IOverridableMember;
           if (overridableMember == null)
               return false;
           IList<OverridableMemberInstance> superMembers = overridableMember.GetRootSuperMembers(true);
            foreach (OverridableMemberInstance superMember in superMembers)
            {
                if (superMember.Member.Module.Name.StartsWith("System"))
                    return true;
            }
           return false;
        }

        private IDeclaration FindDeclarationInFile(IDeclaration[] declarations)
        {
            for (int j = 0; j < declarations.Length; j++)
            {
                IDeclaration declaration1 = declarations[j];
                if (declaration1.GetContainingFile().ProjectItem == this.myProjectFile)
                {
                    return declaration1;
                }
            }
            return null;
        }



        private DocumentRange GetMemberDocumentRange(IDeclaration declaration)
        {
            if (declaration != null)
            {
                DocumentRange range1 = declaration.GetNameDocumentRange();
                if (range1.IsValid)
                    return range1;
            }
            return DocumentRange.InvalidRange;
        }

        private object[] FindUsages(ITypeElement typeElement)
        {
            IFinder finder = typeElement.GetManager().Finder;
            UsagesSearchInfo info1 = this.GetSearchInfo(typeElement);
            SearchPattern pattern1 = info1.SearchPattern;
            IDeclaredElement[] elementArray1 = info1.ElementsToSearch;
            UsagesSearch search = new UsagesSearch(finder, elementArray1, info1, pattern1, info1.Title);
            object[] searchResultAsObj;
            using (ProgressWindow window1 = new ProgressWindow(true))
            {
                bool flag1;
               
                searchResultAsObj = (object[])window1.ExecuteTask(search.Task, "Find unused members", out flag1);
                if (flag1)
                {
                    return null;
                }
            }
            if (searchResultAsObj == null || searchResultAsObj.Length==0)
            {
                WindowUtil.ShowMessageBox("No Usages", "Find Usages", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return null;
            }
            return searchResultAsObj;
        }

        private UsagesSearchInfo GetSearchInfo(IDeclaredElement declaredElement)
        {
            UsagesSearchInfo info1;
            FindUsagesScope scope1 = FindUsagesScope.SOLUTION;
            IProjectModelElement element1 = declaredElement.Module;
            HashSet set1 = new HashSet();
            set1.Add(declaredElement);

            string text1 = "Analyzing Accessibility";
            SearchPattern pattern1 = SearchPattern.FIND_DERIVED_CLASSES_AND_STRUCTS|SearchPattern.FIND_USAGES|SearchPattern.FIND_MEMBER_USAGES;
            ITypeElement element2 = declaredElement.GetContainingType();
            if (element2 != null)
            {
                set1.AddAll(element2.GetMembers(declaredElement.ShortName));
            }
            IDeclaredElement[] elementArray1 = (IDeclaredElement[]) set1.ToArray(typeof(IDeclaredElement));
            info1 = new UsagesSearchInfo(elementArray1, pattern1, scope1, myProjectModelElement, false, text1);
            return info1;

        }

        private UsageStatus GetUsageStatus(IDeclaredElement element, object[] searchResults)
        {
            UsageStatus usageStatus = UsageStatus.Usused;
            for (int i = 0; i < searchResults.Length; i++)
            {
                TextualFindResultItem findResultItem=searchResults[i] as TextualFindResultItem;
                if (findResultItem == null)
                    continue;
                if (findResultItem.Target == element )
                {
                    if (findResultItem.ProjectItem == this.myProjectFile && usageStatus < UsageStatus.AccessedInFile)
                    {
                        usageStatus = UsageStatus.AccessedInFile;
                    }
                    else
                        if (findResultItem.ProjectItem.GetProject() == this.myProjectFile.GetProject()) 
                        {
                            if (usageStatus < UsageStatus.AccessedInProject)
                                usageStatus = UsageStatus.AccessedInProject;
                        }
                        else
                            usageStatus = UsageStatus.AccessedInSolution;
                    
                }
            }
            return usageStatus;
        }

        private AccessibilitySuggestion GetAccessibiltySuggestion(IDeclaredElement element, UsageStatus status)
        {
            IModifiersOwner modifiersOwner=element as IModifiersOwner;
            if(modifiersOwner==null)
                return AccessibilitySuggestion.OK;
            if (status==UsageStatus.Usused)
                return AccessibilitySuggestion.RemoveUnused;
            switch (modifiersOwner.GetAccessRights())
            {
                case AccessRights.INTERNAL:
                    if (status==UsageStatus.AccessedInFile)
                        return AccessibilitySuggestion.CanBePrivate;
                    break;
               case AccessRights.PUBLIC:
                   if (status == UsageStatus.AccessedInFile)
                       return AccessibilitySuggestion.CanBePrivate;
                   if (status == UsageStatus.AccessedInProject)
                       return AccessibilitySuggestion.CanBeInternal;
                    break;
                default:         
                    break;
            }
            return AccessibilitySuggestion.OK;
        }

        public UsageHighlightingEntry[] Entries
        {
            get { return entries; }
        }

        public string StatusBarText
        {
            get { return statusBarText; }
        }
    }
    
    enum UsageStatus
    {
        Usused=0,
        AccessedInFile,
        AccessedInProject,
        AccessedInSolution
    }
    
    enum AccessibilitySuggestion
    {
        RemoveUnused,
        CanBePrivate,
        CanBeInternal,
        OK
    }
}
